@@ -28,8 +28,6 @@ from numpy import (
     isnan,
     percentile,
     zeros,
-    arctan2,
-    arcsin,
     round_,
     nanmean,
     exp,
@@ -45,13 +43,25 @@ from scipy.signal import welch
 from cv2 import filter2D
 from numba import njit, types as numba_types
 from matplotlib.path import Path as BasePath
-from pyproj import Proj
 from pint import UnitRegistry
 from ..observations.observation import EddiesObservations
 from ..eddy_feature import Amplitude, Contours
 from .. import VAR_DESCR
-from ..generic import distance, interp2d_geo, fit_circle, uniform_resample
-from ..poly import poly_contain_poly, winding_number_grid_in_poly, winding_number_poly
+from ..generic import (
+    distance,
+    interp2d_geo,
+    fit_circle,
+    uniform_resample,
+    coordinates_to_local,
+    local_to_coordinates,
+)
+from ..poly import (
+    poly_contain_poly,
+    winding_number_grid_in_poly,
+    winding_number_poly,
+    create_vertice,
+    poly_area,
+)
 
 logger = logging.getLogger("pet")
 
@@ -86,19 +96,11 @@ BasePath.lon = lon
 BasePath.lat = lat
 
 
-@njit(cache=True)
-def prepare_for_kdtree(x_val, y_val):
-    data = empty((x_val.shape[0], 2))
-    data[:, 0] = x_val
-    data[:, 1] = y_val
-    return data
-
-
 @njit(cache=True)
 def uniform_resample_stack(vertices, num_fac=2, fixed_size=None):
     x_val, y_val = vertices[:, 0], vertices[:, 1]
     x_new, y_new = uniform_resample(x_val, y_val, num_fac, fixed_size)
-    data = empty((x_new.shape[0], 2))
+    data = empty((x_new.shape[0], 2), dtype=vertices.dtype)
     data[:, 0] = x_new
     data[:, 1] = y_new
     return data
@@ -124,10 +126,32 @@ def mean_on_regular_contour(
         return values.mean()
 
 
-def fit_circle_path(self):
+def fit_circle_path(self, method="fit"):
     if not hasattr(self, "_circle_params"):
-        self._circle_params = _fit_circle_path(self.vertices)
-    return self._circle_params
+        self._circle_params = dict()
+    if method not in self._circle_params.keys():
+        if method == "fit":
+            self._circle_params["fit"] = _fit_circle_path(self.vertices)
+        if method == "equal_area":
+            self._circle_params["equal_area"] = _circle_from_equal_area(self.vertices)
+    return self._circle_params[method]
+
+
+@njit(cache=True, fastmath=True)
+def _circle_from_equal_area(vertice):
+    lons, lats = vertice[:, 0], vertice[:, 1]
+    # last coordinates == first
+    lon0, lat0 = lons[1:].mean(), lats[1:].mean()
+    c_x, c_y = coordinates_to_local(lons, lats, lon0, lat0)
+    # Some time, edge is only a dot of few coordinates
+    d_lon = lons.max() - lons.min()
+    d_lat = lats.max() - lats.min()
+    if d_lon < 1e-7 and d_lat < 1e-7:
+        # logger.warning('An edge is only define in one position')
+        # logger.debug('%d coordinates %s,%s', len(lons),lons,
+        # lats)
+        return 0, -90, nan, nan
+    return lon0, lat0, (poly_area(create_vertice(c_x, c_y)) / pi) ** 0.5, nan
 
 
 @njit(cache=True, fastmath=True)
@@ -144,10 +168,10 @@ def _fit_circle_path(vertice):
         # logger.debug('%d coordinates %s,%s', len(lons),lons,
         # lats)
         return 0, -90, nan, nan
-    centlon_e, centlat_e, eddy_radius_e, aerr = fit_circle(c_x, c_y)
-    centlon_e, centlat_e = local_to_coordinates(centlon_e, centlat_e, lon0, lat0)
-    centlon_e = (centlon_e - lon0 + 180) % 360 + lon0 - 180
-    return centlon_e, centlat_e, eddy_radius_e, aerr
+    centlon, centlat, eddy_radius, err = fit_circle(c_x, c_y)
+    centlon, centlat = local_to_coordinates(centlon, centlat, lon0, lat0)
+    centlon = (centlon - lon0 + 180) % 360 + lon0 - 180
+    return centlon, centlat, eddy_radius, err
 
 
 @njit(cache=True, fastmath=True)
@@ -195,42 +219,6 @@ def _get_pixel_in_unregular(vertices, x_c, y_c, x_start, x_stop, y_start, y_stop
     return i_x, i_y
 
 
-@njit(cache=True, fastmath=True)
-def coordinates_to_local(lon, lat, lon0, lat0):
-    D2R = pi / 180.0
-    R = 6370997
-    dlon = (lon - lon0) * D2R
-    sin_dlat = sin((lat - lat0) * 0.5 * D2R)
-    sin_dlon = sin(dlon * 0.5)
-    cos_lat0 = cos(lat0 * D2R)
-    cos_lat = cos(lat * D2R)
-    a_val = sin_dlon ** 2 * cos_lat0 * cos_lat + sin_dlat ** 2
-    module = R * 2 * arctan2(a_val ** 0.5, (1 - a_val) ** 0.5)
-
-    azimuth = pi / 2 - arctan2(
-        cos_lat * sin(dlon),
-        cos_lat0 * sin(lat * D2R) - sin(lat0 * D2R) * cos_lat * cos(dlon),
-    )
-    return module * cos(azimuth), module * sin(azimuth)
-
-
-@njit(cache=True, fastmath=True)
-def local_to_coordinates(x, y, lon0, lat0):
-    D2R = pi / 180.0
-    R = 6370997
-    d = (x ** 2 + y ** 2) ** 0.5 / R
-    a = -(arctan2(y, x) - pi / 2)
-    lat = arcsin(sin(lat0 * D2R) * cos(d) + cos(lat0 * D2R) * sin(d) * cos(a))
-    lon = (
-        lon0
-        + arctan2(
-            sin(a) * sin(d) * cos(lat0 * D2R), cos(d) - sin(lat0 * D2R) * sin(lat)
-        )
-        / D2R
-    )
-    return lon, lat / D2R
-
-
 BasePath.fit_circle = fit_circle_path
 
 
@@ -307,6 +295,14 @@ class GridDataset(object):
     def __init__(
         self, filename, x_name, y_name, centered=None, indexs=None, unset=False
     ):
+        """
+        :param str filename: Filename to load
+        :param str x_name: Name of longitude coordinates
+        :param str y_name: Name of latitude coordinates
+        :param bool,None centered: Allow to know how coordinates could be used with pixel
+        :param dict indexs: A dictionary which set indexs to use for non-coordinate dimensions
+        :param bool unset: Set to True to create an empty grid object without file
+        """
         self.dimensions = None
         self.variables_description = None
         self.global_attrs = None
@@ -327,7 +323,7 @@ class GridDataset(object):
         self.interpolators = dict()
         if centered is None:
             logger.warning(
-                "We assume the position of grid is the center corner for %s", filename,
+                "We assume pixel position of grid is center for %s", filename,
             )
         if not unset:
             self.load_general_features()
@@ -469,11 +465,9 @@ class GridDataset(object):
     def copy(self, grid_in, grid_out):
         """
         Duplicate a variable
-        Args:
-            grid_in:
-            grid_out:
 
-        Returns:
+        :param grid_in:
+        :param grid_out:
 
         """
         h_dict = self.variables_description[grid_in]
@@ -585,26 +579,29 @@ class GridDataset(object):
         date,
         step=0.005,
         shape_error=55,
-        array_sampling=50,
+        sampling=50,
         pixel_limit=None,
         precision=None,
         force_height_unit=None,
         force_speed_unit=None,
     ):
         """
+        Compute eddy identification on specified grid
 
-        Args:
-            grid_height:
-            uname:
-            vname:
-            date:
-            step: must be in meter (m)
-            shape_error: must be in percent (%)
-            array_sampling:
-            pixel_limit:
-            precision: must be in meter(m)
+        :param str grid_height: Grid name of height
+        :param str uname: Grid name of u speed component
+        :param str vname: Grid name of v speed component
+        :param datetime.datetime date: Date which will be store in object to date data
+        :param float,int step: Height between two layers in m
+        :param float,int shape_error: Maximal error allow for outter contour in %
+        :param int sampling: Sampling of contour and speed profile
+        :param (int,int),None pixel_limit: Min and max of pixel which must be inside inner and outer contour to be considered like an eddy
+        :param float,None precision: Truncate value at the defined precision in m
+        :param str force_height_unit: Unit to used for height unit
+        :param str force_speed_unit: Unit to used for speed unit
 
-        Returns:
+        :return: Return a list of 2 elements: Anticyclone and Cyclone
+        :rtype: py_eddy_tracker.observations.observation.EddiesObservations
 
         """
         if not isinstance(date, datetime):
@@ -669,6 +666,7 @@ class GridDataset(object):
         # Compute ssh contour
         self.contours = Contours(x, y, data, levels, wrap_x=self.is_circular())
 
+        out_sampling = dict(fixed_size=sampling)
         track_extra_variables = [
             "height_max_speed_contour",
             "height_external_contour",
@@ -708,45 +706,39 @@ class GridDataset(object):
                 )
 
                 # Loop over individual c_s contours (i.e., every eddy in field)
-                for current_contour in contour_paths:
-                    if current_contour.used:
+                for contour in contour_paths:
+                    if contour.used:
                         continue
-                    (
-                        centlon_e,
-                        centlat_e,
-                        eddy_radius_e,
-                        aerr,
-                    ) = current_contour.fit_circle()
+                    # FIXME : center could be not in contour and fit on raw sampling
+                    _, _, _, aerr = contour.fit_circle()
 
                     # Filter for shape
                     if aerr < 0 or aerr > shape_error or isnan(aerr):
                         continue
-                    # Get indices of centroid
-                    # Give only 1D array of lon and lat not 2D data
-                    i_x, i_y = self.nearest_grd_indice(centlon_e, centlat_e)
-                    i_x = self.normalize_x_indice(i_x)
-
-                    # Check if centroid is on define value
-                    if data.mask[i_x, i_y]:
-                        continue
-                    # Test to know cyclone or anticyclone
-                    acyc_not_cyc = data[i_x, i_y] >= cvalues
-                    if anticyclonic_search != acyc_not_cyc:
-                        continue
 
                     # Find all pixels in the contour
-                    i_x_in, i_y_in = current_contour.pixels_in(self)
+                    i_x_in, i_y_in = contour.pixels_in(self)
 
-                    # Maybe limit max must be replace with a maximum of surface
+                    # Check if pixels in contour are masked
+                    if has_masked_value(data.mask, i_x_in, i_y_in):
+                        continue
+
+                    # Test to know cyclone or anticyclone
+                    if has_value(
+                        data, i_x_in, i_y_in, cvalues, below=anticyclonic_search
+                    ):
+                        continue
+
+                    # FIXME : Maybe limit max must be replace with a maximum of surface
                     if (
-                        current_contour.nb_pixel < pixel_limit[0]
-                        or current_contour.nb_pixel > pixel_limit[1]
+                        contour.nb_pixel < pixel_limit[0]
+                        or contour.nb_pixel > pixel_limit[1]
                     ):
                         continue
 
                     # Compute amplitude
                     reset_centroid, amp = self.get_amplitude(
-                        current_contour,
+                        contour,
                         cvalues,
                         data,
                         anticyclonic_search=anticyclonic_search,
@@ -784,82 +776,64 @@ class GridDataset(object):
                         self.contours,
                         centlon_e,
                         centlat_e,
-                        current_contour,
+                        contour,
                         anticyclonic_search,
                         corrected_coll_index,
                         pixel_min=pixel_limit[0],
                     )
 
-                    # Use azimuth equal projection for radius
-                    proj = Proj(
-                        "+proj=aeqd +ellps=WGS84 +lat_0={1} +lon_0={0}".format(
-                            *inner_contour.mean_coordinates
-                        )
-                    )
-                    # First, get position based on innermost
-                    # contour
-                    centx_i, centy_i, _, _ = fit_circle(
-                        *proj(inner_contour.lon, inner_contour.lat)
-                    )
-                    centlon_i, centlat_i = proj(centx_i, centy_i, inverse=True)
-                    # Second, get speed-based radius based on
-                    # contour of max uavg
-                    centx_s, centy_s, eddy_radius_s, aerr_s = fit_circle(
-                        *proj(speed_contour.lon, speed_contour.lat)
-                    )
-                    # Computed again to be coherent with speed_radius, we will be compute in same reference
-                    _, _, eddy_radius_e, aerr_e = fit_circle(
-                        *proj(current_contour.lon, current_contour.lat)
-                    )
-                    centlon_s, centlat_s = proj(centx_s, centy_s, inverse=True)
-
-                    # Instantiate new EddyObservation object (high cost need to be review)
-                    properties = EddiesObservations(
+                    # FIXME : Instantiate new EddyObservation object (high cost need to be review)
+                    obs = EddiesObservations(
                         size=1,
                         track_extra_variables=track_extra_variables,
-                        track_array_variables=array_sampling,
+                        track_array_variables=sampling,
                         array_variables=array_variables,
                     )
 
-                    properties.obs["height_max_speed_contour"] = self.contours.cvalues[
+                    obs.obs["height_max_speed_contour"] = self.contours.cvalues[
                         i_max_speed
                     ]
-                    properties.obs["height_external_contour"] = cvalues
-                    properties.obs["height_inner_contour"] = self.contours.cvalues[
-                        i_inner
-                    ]
+                    obs.obs["height_external_contour"] = cvalues
+                    obs.obs["height_inner_contour"] = self.contours.cvalues[i_inner]
                     array_size = speed_array.shape[0]
-                    properties.obs["nb_contour_selected"] = array_size
+                    obs.obs["nb_contour_selected"] = array_size
                     if speed_array.shape[0] == 1:
-                        properties.obs["uavg_profile"][:] = speed_array[0]
+                        obs.obs["uavg_profile"][:] = speed_array[0]
                     else:
-                        properties.obs["uavg_profile"] = raw_resample(
-                            speed_array, array_sampling
-                        )
-                    properties.obs["amplitude"] = amp.amplitude
-                    properties.obs["radius_s"] = eddy_radius_s
-                    properties.obs["speed_average"] = max_average_speed
-                    properties.obs["radius_e"] = eddy_radius_e
-                    properties.obs["shape_error_e"] = aerr_e
-                    properties.obs["shape_error_s"] = aerr_s
-                    properties.obs["lon"] = centlon_s
-                    properties.obs["lat"] = centlat_s
-                    properties.obs["lon_max"] = centlon_i
-                    properties.obs["lat_max"] = centlat_i
-                    (
-                        properties.obs["contour_lon_e"],
-                        properties.obs["contour_lat_e"],
-                    ) = uniform_resample(
-                        current_contour.lon,
-                        current_contour.lat,
-                        fixed_size=array_sampling,
+                        obs.obs["uavg_profile"] = raw_resample(speed_array, sampling)
+                    obs.obs["amplitude"] = amp.amplitude
+                    obs.obs["speed_average"] = max_average_speed
+                    obs.obs["num_point_e"] = contour.lon.shape[0]
+                    xy_e = uniform_resample(contour.lon, contour.lat, **out_sampling)
+                    obs.obs["contour_lon_e"], obs.obs["contour_lat_e"] = xy_e
+                    obs.obs["num_point_s"] = speed_contour.lon.shape[0]
+                    xy_s = uniform_resample(
+                        speed_contour.lon, speed_contour.lat, **out_sampling
                     )
-                    (
-                        properties.obs["contour_lon_s"],
-                        properties.obs["contour_lat_s"],
-                    ) = uniform_resample(
-                        speed_contour.lon, speed_contour.lat, fixed_size=array_sampling
+                    obs.obs["contour_lon_s"], obs.obs["contour_lat_s"] = xy_s
+
+                    # FIXME : we use a contour without resampling
+                    # First, get position based on innermost contour
+                    centlon_i, centlat_i, _, _ = _fit_circle_path(
+                        create_vertice(inner_contour.lon, inner_contour.lat)
                     )
+                    # Second, get speed-based radius based on contour of max uavg
+                    centlon_s, centlat_s, eddy_radius_s, aerr_s = _fit_circle_path(
+                        create_vertice(*xy_s)
+                    )
+                    # Computed again to use resample contour
+                    _, _, eddy_radius_e, aerr_e = _fit_circle_path(
+                        create_vertice(*xy_e)
+                    )
+
+                    obs.obs["radius_s"] = eddy_radius_s
+                    obs.obs["radius_e"] = eddy_radius_e
+                    obs.obs["shape_error_e"] = aerr_e
+                    obs.obs["shape_error_s"] = aerr_s
+                    obs.obs["lon"] = centlon_s
+                    obs.obs["lat"] = centlat_s
+                    obs.obs["lon_max"] = centlon_i
+                    obs.obs["lat_max"] = centlat_i
                     if aerr > 99.9 or aerr_s > 99.9:
                         logger.warning(
                             "Strange shape at this step! shape_error : %f, %f",
@@ -867,34 +841,31 @@ class GridDataset(object):
                             aerr_s,
                         )
 
-                    eddies.append(properties)
+                    eddies.append(obs)
                     # To reserve definitively the area
                     data.mask[i_x_in, i_y_in] = True
             if len(eddies) == 0:
-                eddies_collection = EddiesObservations(
+                eddies = EddiesObservations(
                     track_extra_variables=track_extra_variables,
-                    track_array_variables=array_sampling,
+                    track_array_variables=sampling,
                     array_variables=array_variables,
                 )
             else:
-                eddies_collection = EddiesObservations.concatenate(eddies)
-            eddies_collection.sign_type = 1 if anticyclonic_search else -1
-            eddies_collection.obs["time"] = (
-                date - datetime(1950, 1, 1)
-            ).total_seconds() / 86400.0
+                eddies = EddiesObservations.concatenate(eddies)
+            eddies.sign_type = 1 if anticyclonic_search else -1
+            eddies.obs["time"] = (date - datetime(1950, 1, 1)).total_seconds() / 86400.0
 
             # normalization longitude between 0 - 360, because storage have an offset on 180
-            eddies_collection.obs["lon_max"] %= 360
-            eddies_collection.obs["lon"] %= 360
-            ref = eddies_collection.obs["lon"] - 180
-            eddies_collection.obs["contour_lon_e"] = (
-                (eddies_collection.obs["contour_lon_e"].T - ref) % 360 + ref
+            eddies.obs["lon_max"] %= 360
+            eddies.obs["lon"] %= 360
+            ref = eddies.obs["lon"] - 180
+            eddies.obs["contour_lon_e"] = (
+                (eddies.obs["contour_lon_e"].T - ref) % 360 + ref
             ).T
-            eddies_collection.obs["contour_lon_s"] = (
-                (eddies_collection.obs["contour_lon_s"].T - ref) % 360 + ref
+            eddies.obs["contour_lon_s"] = (
+                (eddies.obs["contour_lon_s"].T - ref) % 360 + ref
             ).T
-
-            a_and_c.append(eddies_collection)
+            a_and_c.append(eddies)
 
         if in_h_unit is not None:
             for name in [
@@ -1081,7 +1052,7 @@ class UnRegularGridDataset(GridDataset):
     def init_pos_interpolator(self):
         logger.debug("Create a KdTree could be long ...")
         self.index_interp = cKDTree(
-            prepare_for_kdtree(self.x_c.reshape(-1), self.y_c.reshape(-1))
+            create_vertice(self.x_c.reshape(-1), self.y_c.reshape(-1))
         )
 
         logger.debug("... OK")
@@ -1113,6 +1084,7 @@ class UnRegularGridDataset(GridDataset):
                 self.coordinates[0]: x_array[:-1],
                 self.coordinates[1]: y_array[:-1],
             },
+            centered=False,
         )
         regular_grid.bessel_low_filter(grid_name, w_cut, order=1)
         z_filtered = regular_grid.grid(grid_name)
@@ -1154,23 +1126,37 @@ class RegularGridDataset(GridDataset):
     )
 
     def __init__(self, *args, **kwargs):
-        super(RegularGridDataset, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._is_circular = None
 
     def setup_coordinates(self):
-        super(RegularGridDataset, self).setup_coordinates()
+        super().setup_coordinates()
         self.x_size = self.x_c.shape[0]
         self._x_step = (self.x_c[1:] - self.x_c[:-1]).mean()
         self._y_step = (self.y_c[1:] - self.y_c[:-1]).mean()
 
     @classmethod
-    def with_array(cls, coordinates, datas):
+    def with_array(cls, coordinates, datas, variables_description=None, **kwargs):
+        vd = dict() if variables_description is None else variables_description
         x_name, y_name = coordinates[0], coordinates[1]
-        obj = cls("array", x_name, y_name, unset=True, centered=False)
+        obj = cls("array", x_name, y_name, unset=True, **kwargs)
         obj.x_dim = (x_name,)
         obj.y_dim = (y_name,)
+        obj.variables_description = dict()
+        obj.dimensions = {i: v.shape[0] for i, v in datas.items() if i in coordinates}
         for k, v in datas.items():
             obj.vars[k] = v
+            obj.variables_description[k] = dict(
+                attrs=vd.get(k, dict()),
+                args=(k, v.dtype),
+                kwargs=dict(
+                    dimensions=coordinates if k not in coordinates else (k,),
+                    complevel=1,
+                    zlib=True,
+                ),
+                infos=dict(),
+            )
+        obj.global_attrs = dict(history="Grid setup with an array")
         obj.setup_coordinates()
         return obj
 
@@ -1247,57 +1233,26 @@ class RegularGridDataset(GridDataset):
             )
         return self._is_circular
 
-    def kernel_lanczos(self, lat, wave_length, order=1):
-        # Not really operational
-        # wave_length in km
-        # order must be int
+    @staticmethod
+    def check_order(order):
         if order < 1:
             logger.warning("order must be superior to 0")
-        order = ceil(order).astype(int)
-        # Estimate size of kernel
+        return ceil(order).astype(int)
+
+    def get_step_in_km(self, lat, wave_length):
         step_y_km = self.ystep * distance(0, 0, 0, 1) / 1000
         step_x_km = self.xstep * distance(0, lat, 1, lat) / 1000
-        # half size will be multiply with by order
-        half_x_pt, half_y_pt = (
-            ceil(wave_length / step_x_km).astype(int),
-            ceil(wave_length / step_y_km).astype(int),
-        )
-
-        y = arange(
-            lat - self.ystep * half_y_pt * order,
-            lat + self.ystep * half_y_pt * order + 0.01 * self.ystep,
-            self.ystep,
-        )
-        x = arange(
-            -self.xstep * half_x_pt * order,
-            self.xstep * half_x_pt * order + 0.01 * self.xstep,
-            self.xstep,
-        )
-
-        y, x = meshgrid(y, x)
-        dist_norm = distance(0, lat, x, y) / 1000.0 / wave_length
-
-        # sinc(d_x) and sinc(d_y) are windows and bessel function give an equivalent of sinc for lanczos filter
-        kernel = sinc(dist_norm / order) * sinc(dist_norm)
-        kernel[dist_norm > order] = 0
-        return kernel
-
-    def kernel_bessel(self, lat, wave_length, order=1):
-        # wave_length in km
-        # order must be int
-        if order < 1:
-            logger.warning("order must be superior to 0")
-        order = ceil(order).astype(int)
-        # Estimate size of kernel
-        step_y_km = self.ystep * distance(0, 0, 0, 1) / 1000
-        step_x_km = self.xstep * distance(0, lat, 1, lat) / 1000
-        min_wave_length = max(step_x_km * 2, step_y_km * 2)
+        min_wave_length = max(step_x_km, step_y_km) * 2
         if wave_length < min_wave_length:
             logger.error(
                 "Wave_length too short for resolution, must be > %d km",
                 ceil(min_wave_length),
             )
             raise Exception()
+        return step_x_km, step_y_km
+
+    def estimate_kernel_shape(self, lat, wave_length, order):
+        step_x_km, step_y_km = self.get_step_in_km(lat, wave_length)
         # half size will be multiply with by order
         half_x_pt, half_y_pt = (
             ceil(wave_length / step_x_km).astype(int),
@@ -1313,11 +1268,9 @@ class RegularGridDataset(GridDataset):
         x = arange(0, self.xstep * half_x_pt * order + 0.01 * self.xstep, self.xstep)
         y, x = meshgrid(y, x)
         dist_norm = distance(0, lat, x, y) / 1000.0 / wave_length
-        # sinc(d_x) and sinc(d_y) are windows and bessel function give an equivalent of sinc for lanczos filter
-        with errstate(invalid="ignore"):
-            kernel = sinc(dist_norm / order) * j1(2 * pi * dist_norm) / dist_norm
-        kernel[0, half_y_pt * order] = pi
-        kernel[dist_norm > order] = 0
+        return half_x_pt, half_y_pt, dist_norm
+
+    def finalize_kernel(self, kernel, order, half_x_pt, half_y_pt):
         # Symetry
         kernel_ = empty((half_x_pt * 2 * order + 1, half_y_pt * 2 * order + 1))
         kernel_[half_x_pt * order :] = kernel
@@ -1330,6 +1283,33 @@ class RegularGridDataset(GridDataset):
         y_slice = slice(y_valid[0], y_valid[-1] + 1)
         return kernel_[x_slice, y_slice]
 
+    def kernel_lanczos(self, lat, wave_length, order=1):
+        """Not really operational
+        wave_length in km
+        order must be int
+        """
+        order = self.check_order(order)
+        half_x_pt, half_y_pt, dist_norm = self.estimate_kernel_shape(
+            lat, wave_length, order
+        )
+        kernel = sinc(dist_norm / order) * sinc(dist_norm)
+        kernel[dist_norm > order] = 0
+        return self.finalize_kernel(kernel, order, half_x_pt, half_y_pt)
+
+    def kernel_bessel(self, lat, wave_length, order=1):
+        """wave_length in km
+        order must be int
+        """
+        order = self.check_order(order)
+        half_x_pt, half_y_pt, dist_norm = self.estimate_kernel_shape(
+            lat, wave_length, order
+        )
+        with errstate(invalid="ignore"):
+            kernel = sinc(dist_norm / order) * j1(2 * pi * dist_norm) / dist_norm
+        kernel[0, half_y_pt * order] = pi
+        kernel[dist_norm > order] = 0
+        return self.finalize_kernel(kernel, order, half_x_pt, half_y_pt)
+
     def _low_filter(self, grid_name, w_cut, **kwargs):
         """low filtering
         """
@@ -1425,6 +1405,7 @@ class RegularGridDataset(GridDataset):
     def lanczos_high_filter(
         self, grid_name, wave_length, order=1, lat_max=85, **kwargs
     ):
+        logger.warning("It could be not safe to use lanczos filter")
         data_out = self.convolve_filter_with_dynamic_kernel(
             grid_name,
             self.kernel_lanczos,
@@ -1436,6 +1417,7 @@ class RegularGridDataset(GridDataset):
         self.vars[grid_name] -= data_out
 
     def lanczos_low_filter(self, grid_name, wave_length, order=1, lat_max=85, **kwargs):
+        logger.warning("It could be not safe to use lanczos filter")
         data_out = self.convolve_filter_with_dynamic_kernel(
             grid_name,
             self.kernel_lanczos,
@@ -1505,7 +1487,8 @@ class RegularGridDataset(GridDataset):
             pws.append(pw)
         if nb_invalid:
             logger.warning("%d/%d columns invalid", nb_invalid, i + 1)
-        lat_content = 1 / f, array(pws).mean(axis=0)
+        with errstate(divide="ignore"):
+            lat_content = 1 / f, array(pws).mean(axis=0)
 
         # Lon spectrum
         fs, pws = list(), list()
@@ -1534,7 +1517,8 @@ class RegularGridDataset(GridDataset):
                 for f, pw in zip(fs, pws)
             ]
         ).mean(axis=0)
-        lon_content = 1 / f_interp, pw_m
+        with errstate(divide="ignore"):
+            lon_content = 1 / f_interp, pw_m
         if ref is None:
             return lon_content, lat_content
         else:
@@ -1777,16 +1761,19 @@ class RegularGridDataset(GridDataset):
     def interp(self, grid_name, lons, lats):
         """
         Compute z over lons, lats
-        Args:
-            grid_name: Grid which will be interp
-            lons: new x
-            lats: new y
 
-        Returns:
-            new z
+        :param str grid_name: Grid which will be interp
+        :param lons: new x
+        :param lats: new y
+
+        :return: new z
         """
         g = self.grid(grid_name)
-        return interp2d_geo(self.x_c, self.y_c, g, g.mask, lons, lats)
+        if len(g.mask.shape):
+            m = g.mask
+        else:
+            m = ones(g.shape) if g.mask else zeros(g.shape)
+        return interp2d_geo(self.x_c, self.y_c, g, m, lons, lats)
 
 
 @njit(cache=True, fastmath=True)
@@ -1880,3 +1867,23 @@ def _nearest_grd_indice(x, y, x0, y0, xstep, ystep):
         numba_types.int32(round(((x - x0[0]) % 360.0) / xstep)),
         numba_types.int32(round((y - y0[0]) / ystep)),
     )
+
+
+@njit(cache=True)
+def has_masked_value(grid, i_x, i_y):
+    for i, j in zip(i_x, i_y):
+        if grid[i, j]:
+            return True
+    return False
+
+
+@njit(cache=True)
+def has_value(grid, i_x, i_y, value, below=False):
+    for i, j in zip(i_x, i_y):
+        if below:
+            if grid[i, j] < value:
+                return True
+        else:
+            if grid[i, j] > value:
+                return True
+    return False
