@@ -48,6 +48,8 @@ from numpy import (
     ceil,
     arange,
     histogram2d,
+    linspace,
+    sin,
 )
 from netCDF4 import Dataset
 from datetime import datetime
@@ -56,37 +58,22 @@ from Polygon import Polygon
 from pint import UnitRegistry
 from pint.errors import UndefinedUnitError
 from tokenize import TokenError
+from tarfile import ExFileObject
 from matplotlib.path import Path as BasePath
 from .. import VAR_DESCR, VAR_DESCR_inv
-from ..generic import distance_grid, distance, flatten_line_matrix, wrap_longitude
-from ..poly import bbox_intersection, common_area, create_vertice
+from ..generic import (
+    distance_grid,
+    distance,
+    flatten_line_matrix,
+    wrap_longitude,
+    local_to_coordinates,
+    reverse_index,
+)
+from ..poly import bbox_intersection, vertice_overlap, create_vertice
 
 logger = logging.getLogger("pet")
 
 
-@njit(cache=True, fastmath=True)
-def shifted_ellipsoid_degrees_mask(lon0, lat0, lon1, lat1, minor=1.5, major=1.5):
-    # c = (major ** 2 - minor ** 2) ** .5 + major
-    c = major
-    major = minor + 0.5 * (major - minor)
-    # r=.5*(c-c0)
-    # a=c0+r
-    # Focal
-    f_right = lon0
-    f_left = f_right - (c - minor)
-    # Ellipse center
-    x_c = (f_left + f_right) * 0.5
-
-    nb_0, nb_1 = lat0.shape[0], lat1.shape[0]
-    m = empty((nb_1, nb_0), dtype=numba_types.bool_)
-    for i in range(nb_1):
-        dy = lat1[i] - lat0
-        dx = (lon1[i] - x_c + 180) % 360 - 180
-        d_normalize = dx ** 2 / major ** 2 + dy ** 2 / minor ** 2
-        m[i] = d_normalize < 1.0
-    return m.T
-
-
 @njit(cache=True, fastmath=True)
 def shifted_ellipsoid_degrees_mask2(lon0, lat0, lon1, lat1, minor=1.5, major=1.5):
     """
@@ -151,6 +138,8 @@ class EddiesObservations(object):
         "shape_error_e",
         "shape_error_s",
         "nb_contour_selected",
+        "num_point_e",
+        "num_point_s",
         "height_max_speed_contour",
         "height_external_contour",
         "height_inner_contour",
@@ -247,6 +236,23 @@ class EddiesObservations(object):
         new.observations["type_cyc"] = self.sign_type
         return new
 
+    def circle_contour(self):
+        angle = radians(linspace(0, 360, self.track_array_variables))
+        x_norm, y_norm = cos(angle), sin(angle)
+        for i, obs in enumerate(self):
+            r_s, r_e, x, y = (
+                obs["radius_s"],
+                obs["radius_e"],
+                obs["lon"],
+                obs["lat"],
+            )
+            obs["contour_lon_s"], obs["contour_lat_s"] = local_to_coordinates(
+                x_norm * r_s, y_norm * r_s, x, y
+            )
+            obs["contour_lon_e"], obs["contour_lat_e"] = local_to_coordinates(
+                x_norm * r_e, y_norm * r_e, x, y
+            )
+
     @property
     def dtype(self):
         """Return dtype to build numpy array
@@ -386,9 +392,11 @@ class EddiesObservations(object):
             raw_data=eddies.raw_data,
         )
 
-    def index(self, index):
+    def index(self, index, reverse=False):
         """Return obs from self at the index
         """
+        if reverse:
+            index = reverse_index(index, len(self))
         size = 1
         if hasattr(index, "__iter__"):
             size = len(index)
@@ -406,8 +414,11 @@ class EddiesObservations(object):
 
     @classmethod
     def load_file(cls, filename, **kwargs):
-        end = b".zarr" if isinstance(filename, bytes) else ".zarr"
-        if filename.endswith(end):
+        filename_ = (
+            filename.filename if isinstance(filename, ExFileObject) else filename
+        )
+        end = b".zarr" if isinstance(filename_, bytes) else ".zarr"
+        if filename_.endswith(end):
             return cls.load_from_zarr(filename, **kwargs)
         else:
             return cls.load_from_netcdf(filename, **kwargs)
@@ -514,9 +525,14 @@ class EddiesObservations(object):
         cls, filename, raw_data=False, remove_vars=None, include_vars=None
     ):
         array_dim = "NbSample"
-        if not isinstance(filename, str):
+        if isinstance(filename, bytes):
             filename = filename.astype(str)
-        with Dataset(filename) as h_nc:
+        if isinstance(filename, ExFileObject):
+            filename.seek(0)
+            args, kwargs = ("in-mem-file",), dict(memory=filename.read())
+        else:
+            args, kwargs = (filename,), dict()
+        with Dataset(*args, **kwargs) as h_nc:
             var_list = list(h_nc.variables.keys())
             if include_vars is not None:
                 var_list = [i for i in var_list if i in include_vars]
@@ -652,16 +668,15 @@ class EddiesObservations(object):
     ):
         """
         Filled virtual obs (C)
-        Args:
-            previous_obs: previous obs from current (A)
-            current_obs: previous obs from virtual (B)
-            obs_to_extend:
-            dead_track:
-            nb_next:
-            model:
 
-        Returns:
-            New position C = B + AB
+        :param previous_obs: previous obs from current (A)
+        :param current_obs: previous obs from virtual (B)
+        :param obs_to_extend:
+        :param dead_track:
+        :param nb_next:
+        :param model:
+
+        :return: New position C = B + AB
         """
         next_obs = VirtualEddiesObservations(
             size=nb_next,
@@ -701,35 +716,36 @@ class EddiesObservations(object):
         return next_obs
 
     @staticmethod
-    def intern(flag):
-        return (
-            ("contour_lon_s", "contour_lat_s")
-            if flag
-            else ("contour_lon_e", "contour_lat_e")
-        )
+    def intern(flag, public_label=False):
+        if flag:
+            labels = "contour_lon_s", "contour_lat_s"
+        else:
+            labels = "contour_lon_e", "contour_lat_e"
+        if public_label:
+            labels = [VAR_DESCR[label]["nc_name"] for label in labels]
+        return labels
 
-    def match(self, other, intern=False):
+    def match(self, other, intern=False, cmin=0):
         """return index and score compute with area
         """
         x_name, y_name = self.intern(intern)
         i, j = bbox_intersection(
             self[x_name], self[y_name], other[x_name], other[y_name]
         )
-        c = common_area(
+        c = vertice_overlap(
             self[x_name][i], self[y_name][i], other[x_name][j], other[y_name][j]
         )
-        m = c > 0
+        m = c > cmin
         return i[m], j[m], c[m]
 
     @classmethod
     def cost_function_common_area(cls, xy_in, xy_out, distance, intern=False):
         """ How does it work on x bound ?
-        Args:
-            xy_in:
-            xy_out:
-            distance:
-            intern:
-        Returns:
+
+        :param xy_in:
+        :param xy_out:
+        :param distance:
+        :param bool intern:
 
         """
         x_name, y_name = cls.intern(intern)
@@ -1238,14 +1254,16 @@ class EddiesObservations(object):
             x = (x - ref) % 360 + ref
         return ax.scatter(x, self.latitude, c=self[name] * factor, **kwargs)
 
-    def display(self, ax, ref=None, extern_only=False, intern_only=False, **kwargs):
+    def display(
+        self, ax, ref=None, extern_only=False, intern_only=False, nobs=True, **kwargs
+    ):
         if not extern_only:
             lon_s = flatten_line_matrix(self.obs["contour_lon_s"])
             lat_s = flatten_line_matrix(self.obs["contour_lat_s"])
         if not intern_only:
             lon_e = flatten_line_matrix(self.obs["contour_lon_e"])
             lat_e = flatten_line_matrix(self.obs["contour_lat_e"])
-        if "label" in kwargs:
+        if nobs and "label" in kwargs:
             kwargs["label"] += " (%s observations)" % len(self)
         kwargs_e = kwargs.copy()
         if not extern_only:
@@ -1269,14 +1287,23 @@ class EddiesObservations(object):
 
         regular_grid = RegularGridDataset.with_array(
             coordinates=("lon", "lat"),
-            datas=dict(count=grid, lon=x_bins[:-1], lat=y_bins[:-1],),
+            datas=dict(
+                count=grid,
+                lon=(x_bins[1:] + x_bins[:-1]) / 2,
+                lat=(y_bins[1:] + y_bins[:-1]) / 2,
+            ),
+            variables_description=dict(
+                count=dict(long_name="Number of times pixel is in eddies")
+            ),
+            centered=True,
         )
         if center:
             x, y = (self.longitude - x0) % 360 + x0, self.latitude
             grid[:] = histogram2d(x, y, (x_bins, y_bins))[0]
             grid.mask = grid.data == 0
         else:
-            x, y = (self[x_name] - x0) % 360 + x0, self[y_name]
+            x_ref = ((self.longitude - x0) % 360 + x0 - 180).reshape(-1, 1)
+            x, y = (self[x_name] - x_ref) % 360 + x_ref, self[y_name]
             for x_, y_ in zip(x, y):
                 i, j = BasePath(create_vertice(x_, y_)).pixels_in(regular_grid)
                 grid_count_(grid, i, j)
@@ -1316,6 +1343,6 @@ class VirtualEddiesObservations(EddiesObservations):
 
     @property
     def elements(self):
-        elements = super(VirtualEddiesObservations, self).elements
+        elements = super().elements
         elements.extend(["track", "segment_size", "dlon", "dlat"])
         return list(set(elements))
